package crontab

import (
	"context"
	"fmt"
	"kpt-pasture/model"
	"kpt-pasture/module/backend"
	"kpt-pasture/util"
	"time"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
	"gitee.com/xuyiping_admin/pkg/logger/zaplog"
	"gitee.com/xuyiping_admin/pkg/xerr"
	"go.uber.org/zap"
)

const (
	UpdateCowInfo    = "UpdateCowInfo"
	ImmunizationPlan = "ImmunizationPlan"
	SameTimePlan     = "SameTimePlan"
	WorkOrderMaster  = "WorkOrderMaster"
)

// GenerateAsynqWorkOrder 异步生成工作单
func (e *Entry) GenerateAsynqWorkOrder() error {
	if ok := e.IsExistCrontabLog(WorkOrderMaster); ok {
		return nil
	}
	defer func() {
		e.CreateCrontabLog(WorkOrderMaster)
	}()
	workOrderList := make([]*model.WorkOrderMaster, 0)
	if err := e.DB.Where("is_show = ?", pasturePb.IsShow_Ok).Find(&workOrderList).Error; err != nil {
		return err
	}
	for _, workOrder := range workOrderList {
		timeUnix, err := util.ConvertParseLocalUnix(workOrder.ExecTime)
		if timeUnix <= 0 || err != nil {
			zaplog.Error("crontab", zap.Any("GenerateWorkOrder", err), zap.Any("execTime", workOrder.ExecTime))
			continue
		}
		execTime := time.Now().Unix() - timeUnix

		task := model.NewTaskWorkOrderPayload(workOrder.Id, time.Duration(execTime)*time.Second)
		if _, err = e.AsynqClient.CtxEnqueue(context.Background(), task); err != nil {
			zaplog.Error("PushMessage CtxEnqueue", zap.Any("Err", err))
		}
	}
	return nil
}

// UpdateCowInfo 牛只基本信息维护
func (e *Entry) UpdateCowInfo() error {
	cowList := make([]*model.Cow, 0)
	if err := e.DB.Where("is_remove = ?", pasturePb.IsShow_Ok).Find(&cowList).Error; err != nil {
		return err
	}
	if ok := e.IsExistCrontabLog(UpdateCowInfo); ok {
		return nil
	}
	defer func() {
		e.CreateCrontabLog(UpdateCowInfo)
	}()
	for _, cow := range cowList {
		if err := e.DB.Model(new(model.Cow)).Where("id = ?", cow.Id).Updates(map[string]interface{}{
			"day_age":       cow.GetDayAge(),
			"calving_at":    cow.GetCalvingAge(),
			"pregnancy_age": cow.GetDaysPregnant(),
			"admission_age": cow.GetAdmissionAge(),
			"abortion_age":  cow.GetAbortionAge(),
		}).Error; err != nil {
			zaplog.Error("Crontab", zap.Any("UpdateCowDayAge", err))
		}
	}
	return nil
}

// ImmunizationPlan 免疫计划，生成工作单
func (e *Entry) ImmunizationPlan() error {
	if ok := e.IsExistCrontabLog(ImmunizationPlan); ok {
		return nil
	}
	planList := make([]*model.ImmunizationPlan, 0)
	if err := e.DB.Where("is_show = ?", pasturePb.IsShow_Ok).Find(&planList).Error; err != nil {
		return xerr.WithStack(err)
	}
	var todayCount int32 = 0
	defer func() {
		var count int64 = 0
		if err := e.DB.Model(new(model.CowImmunizationPlan)).Where("status = ?", pasturePb.IsShow_Ok).Count(&count).Error; err != nil {
			zaplog.Error("Crontab", zap.Any("ImmunizationPlanDefer", err))
		}
		todayCount += int32(count)
		if todayCount > 0 {
			e.CreatedCalendar(pasturePb.CalendarType_Immunisation, todayCount)
		}
		e.CreateCrontabLog(ImmunizationPlan)
	}()

	for _, plan := range planList {
		if plan == nil {
			continue
		}
		cowList := make([]*model.Cow, 0)
		pref := e.DB.Table(fmt.Sprintf("%s as a", new(model.ImmunizationPlan).TableName())).
			Select("a.*").
			Where("a.is_show = ?", pasturePb.IsShow_Ok)
		if plan.CowType > 0 {
			pref.Where("a.cow_type = ?", plan.CowType)
		}

		switch plan.Conditions {
		case pasturePb.ImmunizationConditions_Days_Age:
			pref.Where("a.day_age = ?", plan.Value)
		case pasturePb.ImmunizationConditions_Days_After_Delivery:
			pref.Where("a.calving_age = ?", plan.Value)
		case pasturePb.ImmunizationConditions_Days_Of_Pregnancy:
			pref.Where("a.pregnancy_age = ?", plan.Value).
				Where("a.is_pregnant = ?", pasturePb.IsShow_Ok)
		case pasturePb.ImmunizationConditions_Month:
			// todo 待实现月份
		case pasturePb.ImmunizationConditions_Admission_Days:
			pref.Where("a.admission_age = ?", plan.Value)
		case pasturePb.ImmunizationConditions_Other_Vaccine_After:
			if plan.ImmunizationPlanId > 0 {
				pref.Joins("INNER JOIN immunization_plan_cow as b ON b.immunization_plan_id = ?", plan.ImmunizationPlanId).
					Where("b.cow_id = a.id").
					Where("DATE_ADD(b.reality_time, INTERVAL ? DAY) = ?", plan.Value, time.Now().Format(model.LayoutDate2)).
					Where("b.status = ?", pasturePb.IsShow_Ok)
			}
		}

		if err := pref.Find(&cowList).Error; err != nil {
			return xerr.WithStack(err)
		}
		if len(cowList) <= 0 {
			continue
		}
		todayCount += int32(len(cowList))
		newImmunizationPlanCowList := model.NewCowImmunizationPlanList(cowList, plan)
		if err := e.DB.Create(newImmunizationPlanCowList).Error; err != nil {
			zaplog.Error("ImmunizationPlan", zap.Any("newImmunizationPlanCowList", err), zap.Any("plan", plan), zap.Any("cowList", cowList))
		}
	}
	zaplog.Info("ImmunizationPlan", zap.Any("todayCount", todayCount))
	return nil
}

// SameTimePlan 同期计划，生成工作单
func (e *Entry) SameTimePlan() error {
	if ok := e.IsExistCrontabLog(SameTimePlan); ok {
		return nil
	}

	sameTimeList := make([]*model.SameTime, 0)
	if err := e.DB.Where("is_show = ?", pasturePb.IsShow_Ok).Find(&sameTimeList).Error; err != nil {
		return xerr.WithStack(err)
	}

	// 更新日历里面的数据
	var todayCount int32 = 0
	defer func() {
		if todayCount > 0 {
			e.CreatedCalendar(pasturePb.CalendarType_Immunisation, todayCount)
		}
		e.CreateCrontabLog(SameTimePlan)
	}()

	currWeek := time.Now().Weekday()
	for _, sameTime := range sameTimeList {
		if sameTime == nil {
			continue
		}

		if time.Weekday(sameTime.WeekType) != currWeek {
			continue
		}

		cowList := make([]*model.Cow, 0)
		pref := e.DB.Where("is_remove = ?", pasturePb.IsShow_Ok)

		if sameTime.CowType == pasturePb.SameTimeCowType_Breeding_Calf {
			pref.Where("calving_age >= ?", sameTime.PostpartumDaysStart).
				Where("calving_age <= ?", sameTime.PostpartumDaysEnd).
				Where("is_pregnant = ?", pasturePb.IsShow_No)
		}

		if sameTime.CowType == pasturePb.SameTimeCowType_Empty {
			pref.Where(
				e.DB.Where("breed_status = ?", pasturePb.BreedStatus_Empty).
					Or("breed_status = ?", pasturePb.BreedStatus_Abort),
			).Where("is_pregnant = ?", pasturePb.IsShow_No)
		}

		if err := pref.Find(&cowList).Error; err != nil {
			zaplog.Error("crontab", zap.Any("SameTimePlan", err), zap.Any("plan", sameTime))
			return xerr.WithStack(err)
		}

		if len(cowList) <= 0 {
			continue
		}

		currCount, err := e.GenerateCalendarBySameTimePlan(cowList, sameTime)
		if err != nil {
			zaplog.Error("crontab",
				zap.Any("GenerateCalendarBySameTimePlan", err),
				zap.Any("cowList", cowList),
				zap.Any("plan", sameTime),
			)
			continue
		}
		todayCount += currCount
	}
	return nil
}

// UpdateSameTime 更新每天同情数据
func (e *Entry) UpdateSameTime() error {
	calendarTypeList := backend.CalendarTypeEnumList()
	showDay := time.Now().Format(model.LayoutDate2)
	for _, v := range calendarTypeList {
		count := int64(0)
		if err := e.DB.Model(new(model.EventCowSameTime)).
			Where("same_time_type = ?", v.Value).
			Where("status = ?", pasturePb.IsShow_No).
			Count(&count); err != nil {
			zaplog.Error("crontab", zap.Any("UpdateSameTime", err), zap.Any("count", count))
		}
		if count >= 0 {
			continue
		}
		isExist := int64(0)
		if err := e.DB.Model(new(model.Calendar)).
			Where("calendar_type = ?", v.Value).
			Where("show_day = ?", showDay).
			Count(&isExist).Error; err != nil {
			continue
		}
		if isExist <= 0 {
			continue
		}
		if err := e.DB.Model(new(model.Calendar)).
			Where("calendar_type = ?", v.Value).
			Where("show_day = ?", showDay).
			Updates(map[string]interface{}{
				"count": count,
			}).Error; err != nil {
			continue
		}
	}
	return nil
}

// PregnancyCheck 妊娠期检查
func (e *Entry) PregnancyCheck() error {
	return nil
}

// DiseaseCheck 疾病检查
func (e *Entry) DiseaseCheck() error {
	return nil
}
